#!/usr/bin/python -tt

import os
import sys
import subprocess

class Cores(object):
    def __init__(self, coreid):
        self.id = coreid
        self.cpus = set()
        self.inuse = []

    def add(self, cpu_num):
        self.cpus.add(cpu_num)

    def ncpus(self):
        return len(self.cpus)

    def getcpus(self):
        return list(self.cpus)

    def get(self):
        for c in self.cpus:
            if c not in self.inuse:
                self.inuse.append(c)
                return c
        return -1;

    def put(self, core):
        self.inuse.remove(core)

    def __str__(self):
        return "core %d: %s" % (self.id, str(self.cpus))


class Nodes(object):
    def __init__(self, nodeid):
        self.cores = {}
        self.id = int(nodeid)
        self.inuse = []

    def add(self, core, cpu):
        if not self.cores.has_key(core):
            self.cores[core] = Cores(core)
        self.cores[core].add(cpu)

    def getcpu(self):
        for coreid in self.cores.keys():
            cpu = self.cores[coreid].get()
            if cpu > -1:
                return cpu
        raise RuntimeError, "all cpus in use!"

    def putcpu(self, cpu):
        for coreid in self.cores.keys():
            core = self.cores[coreid]
            if cpu in core.cpus:
                core.put(cpu)
                return

    def allcpus(self):
        cpus = []
        for coreid in self.cores.keys():
            cpus += self.cores[coreid].getcpus()
        cpus.sort()
        return cpus

    def __str__(self):
        out = [ str(self.cores[c]) for c in self.cores.keys() ]
        return "node: %d, %d cores (%s)" % (self.id, len(self.cores), out)

if __name__ == "__main__":
    p = subprocess.Popen(['lscpu', '--parse=node,core,cpu'], stdout=subprocess.PIPE)
    lines = p.stdout.readlines()

    nodes = {}
    for l in lines:
        if l.startswith('#'): continue
        l = l.strip()
        (node, core, cpu) = [int(x) for x in l.split(',')]
        if not nodes.has_key(node):
            nodes[node] = Nodes(node)
        nodes[node].add(core,cpu)

    realtime = []
    for nodeid in nodes.keys():
        print nodes[nodeid]
        print "Node %d: %s" % (int(nodeid), nodes[nodeid].allcpus())
        realtime.append(nodes[nodeid].getcpu())

    print "Realtime cores: %s" % realtime
